//
// Kör alakú sakktábla
// Felhasználási feltételek: Nevezd meg! - Így add tovább! (Creative Commons)
// Uray M. János
// 2013-2014
//

#include <iostream>
#include <string>
#include <vector>
#include <cmath>
#include <cstring>
#include <utility>

using namespace std;

float const pi = 3.1415926535;

int main (int n_args, char const* args[]) {
	if (n_args < 5) {
		cerr << "Használat:\n";
		cerr << args[0] << " <keresztbe (db)> <hosszában (db)> <külső sugár (pt)> <belső sugár (pt)> <paraméterek>\n";
		cerr << "További paraméterek:\n";
		cerr << "   -edge=<szegély (pt)>\n";
		cerr << "   -margin=<margó (pt)>\n";
		cerr << "   -a1=(white|black)\n";
		cerr << "   -mark=<hányanként>\n";
		cerr << "Például: 4 16 384 128 -edge=4\n";
		return 1;
	}
	//
	// paraméterek
	//
	vector<string> sargs(args, args + n_args);
	int rings = stoi(sargs[1]);
	int files = stoi(sargs[2]);
	int outer = stoi(sargs[3]);
	int inner = stoi(sargs[4]);
	int edge = 0, margin = 0;
	bool back = false;
	int mark = 0;
	for (unsigned i = 5; i < sargs.size(); i++) {
		if (std::strncmp(sargs[i].c_str(), "-edge=", 6) == 0) {
			edge = stoi(sargs[i].substr(6));
		} else if (std::strncmp(sargs[i].c_str(), "-margin=", 8) == 0) {
			margin = stoi(sargs[i].substr(8));
		} else if (std::strncmp(sargs[i].c_str(), "-a1=", 4) == 0) {
			back = sargs[i][4] == 'w';
		} else if (std::strncmp(sargs[i].c_str(), "-mark=", 6) == 0) {
			mark = stoi(sargs[i].substr(6));
		} else {
			return std::cerr << "Hibás paraméter: " << sargs[i] << "\n", 2;
		}
	}
	margin = std::max(margin, edge);
	//
	// kiírás
	//
	cout << "<?xml version='1.0' encoding='UTF-8' standalone='no'?>\n";
	cout << "<!--\n";
	cout << "\tA képet a circular.cpp generálta a következő paraméterekkel:\n";
	for (unsigned i = 1; i < sargs.size(); i++)
		cout << (i == 1 ? "\t" : " ") << sargs[i];
	cout << "\n";
	cout << "\tFelhasználási feltételek: Nevezd meg! - Így add tovább! (Creative Commons)\n";
	cout << "\tUray M. János\n";
	cout << "\t2013-2014\n";
	cout << "-->\n";
	cout << "<!DOCTYPE svg PUBLIC '-//W3C//DTD SVG 1.1//EN' 'http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd'>\n";
	cout << "<svg ";
	cout << "width='" << 2*outer + 2*margin << "' ";
	cout << "height='" << 2*outer + 2*margin << "' ";
	cout << "viewBox='" << -(outer + margin) << " " << -(outer + margin) << " " << 2*(outer + margin) << " " << 2*(outer + margin) << "' ";
	cout << "xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'>\n";
	cout << "\t\n";
	if (edge) {
		cout << "\t<!-- keret -->\n";
		cout << "\t<circle fill='none' stroke='#666' stroke-width='" << 2*edge << "' cx='0' cy='0' r='" << outer << "'/>\n";
		cout << "\t<circle fill='none' stroke='#666' stroke-width='" << 2*edge << "' cx='0' cy='0' r='" << inner << "'/>\n";
		cout << "\t\n";
	}
	cout << "\t<!-- mezők -->\n";
	cout << "\t<path fill='#AAA' fill-rule='evenodd' d='M " << outer << ",0 A " << outer << "," << outer << " 0 0,0 0,-" << outer << " A " << outer << "," << outer << " 0 1,0 " << outer << ",0 M " << inner << ",0 A " << inner << "," << inner << " 0 1,1 0,-" << inner << " A " << inner << "," << inner << " 0 0,1 " << inner << ",0'/>\n";
	float const ring_size = float(outer - inner) / rings; 
	for (int y = 0; y < rings; y++) {
		float r1 = outer - y * ring_size;
		float r2 = outer - (y + 1) * ring_size;
		for (int x = 0; x < files; x++) {
			float a1 = x * 2*pi / files;
			float a2 = (x + 1) * 2*pi / files;
			if ((x + y) % 2 == !back) {
				cout << "\t<path fill='#FFF' d='M " << sin(a1)*r1 << "," << cos(a1)*r1 << " A " << r1 << "," << r1 << " 0 0,0 " << sin(a2)*r1 << "," << cos(a2)*r1 << " L " << sin(a2)*r2 << "," << cos(a2)*r2 << " A " << r2 << "," << r2 << " 0 0,1 " << sin(a1)*r2 << "," << cos(a1)*r2 << "'/>\n";
			}
		}
	}
	cout << "\t\n";
	if (mark && edge) {
		cout << "\t<!-- jelölés -->\n";
		for (int x = 0; x < files; x += mark) {
			float angle = x * 2*pi / files;
			cout << "\t<line " <<
				"stroke='#666' " <<
				"stroke-width='" << edge/2 << "' " <<
				"stroke-dasharray='" << ring_size*3/16 << ", " << ring_size*5/16 << "' " <<
				"stroke-dashoffset='" << ring_size*3/32 << "' " <<
				"stroke-linecap='round' " <<
				"x1='" << sin(angle) * inner << "' " <<
				"y1='" << cos(angle) * inner << "' " <<
				"x2='" << sin(angle) * outer << "' " <<
				"y2='" << cos(angle) * outer << "' " <<
			"/>\n";
		}
		cout << "\t\n";
	}
	cout << "</svg>\n";
}

