//
// Hatszögrácsos sakktábla
// Felhasználási feltételek: Nevezd meg! - Így add tovább! (Creative Commons)
// Uray M. János
// 2013-2014
//

#include <iostream>
#include <vector>
#include <string>
#include <cstring>
#include <cmath>
#include <list>
#include <cassert>
#include <sstream>
#include <utility>

using namespace std;

int main (int n_args, char const* args[]) {
	if (n_args < 3) {
		cerr << "Használat:\n";
		cerr << args[0] << " <mezőméret (pt)> <méret> <paraméterek>\n";
		cerr << "<méret>: -side=<db> vagy -sides=<a>,<b>,<c>,<d>,<e>,<f>\n";
		cerr << "További paraméterek:\n";
		cerr << "   -edge=<szegély (pt)>\n";
		cerr << "   -margin=<x-margó (pt)>,<y-margó (pt)>\n";
		cerr << "   -color-order=(0|1|2|3|4|5)\n";
		cerr << "   -fields=<mezőfelsorolás (0|1-sorozat)>\n";
		cerr << "Például: 64 -size=6 -edge=4\n";
		return 1;
	}
	//
	// paraméterek
	//
	vector<string> sargs(args, args + n_args);
	int const size = stoi(sargs[1]);
	double const hexside = size / sqrt(3);
	int files, rows;
	vector<vector<bool>> fields;
	if (strncmp(sargs[2].c_str(), "-side=", 6) == 0) {
		int side = stoi(sargs[2].substr(6));
		files = 2*side - 1;
		rows = 4*side - 3;
		fields.resize(rows);
		for (int y = 0; y < rows; y++) {
			fields[y].resize(files, false);
			int len = y < side ? y + 1 : y > 3*side - 3 ? rows - y : (y - side)%2 ? side : side - 1;
			for (int i = 0; i < len; i++)
				fields[y][side - len + 2*i] = true;
		}
	} else if (strncmp(sargs[2].c_str(), "-sides=", 7) == 0) {
		istringstream iss(sargs[2].substr(7));
		char comma;
		int a, b, c, d, e, f;
		iss >> a >> comma >> b >> comma >> c >> comma >> d >> comma >> e >> comma >> f;
		if (iss.fail() || !iss.eof() || comma != ',')
			return cerr << "-sides: hibás paraméter\n", 2;
		if (a + 2*b + c != d + 2*e + f || a + f != c + d)
			return cerr << "-sides: nem adnak hatszöget\n", 2;
		files = a + f - 1;
		rows = a + 2*b + c - 3;
		fields.resize(rows);
		for (int y = 0; y < rows; y++)
			fields[y].resize(files, false);
		for (int y = 0; y < rows; y++) {
			for (int x = 0; x < files; x++) {
				if (((x + y) - (a - 1)) % 2)
					continue;
				if (x + y < a - 1)
					continue;
				if ((files - x - 1) + y < f - 1)
					continue;
				if (x + (rows - y - 1) < c - 1)
					continue;
				if ((files - x - 1) + (rows - y - 1) < d - 1)
					continue;
				fields[y][x] = true;
			}
		}
	} else {
		return cerr << "Hibás második paraméter: " << args[2] << "\n", 2;
	}
	int edge = 0, margin_x = 0, margin_y = 0;
	int color_order = 0;
	for (unsigned i = 3; i < sargs.size(); i++) {
		if (strncmp(sargs[i].c_str(), "-edge=", 6) == 0) {
			edge = stoi(sargs[i].substr(6));
		} else if (strncmp(sargs[i].c_str(), "-margin=", 8) == 0) {
			size_t idx;
			margin_x = margin_y = stoi(sargs[i].substr(8), &idx);
			if (sargs[i].size() > 8 + idx)
				margin_y = stoi(sargs[i].substr(8 + idx + 1));
		} else if (strncmp(sargs[i].c_str(), "-color-order=", 13) == 0) {
			color_order = stoi(sargs[i].substr(13));
		} else if (strncmp(sargs[i].c_str(), "-fields=", 8) == 0) {
			unsigned j = 8;
			for (int y = 0; y < rows; y++) {
				for (int x = 0; x < files; x++) {
					if (fields[y][x]) {
						if (j == sargs[i].size())
							return cerr << "-fields: Nem megfelelő számú mező.\n", 2;
						fields[y][x] = sargs[i][j] != '0';
						j++;
					}
				}
			}
			if (j != sargs[i].size())
				return cerr << "-fields: Nem megfelelő számú mező.\n", 2;
		} else {
			return cerr << "Hibás paraméter: " << sargs[i] << "\n", 2;
		}
	}
	margin_x = std::max(margin_x, (int)(ceil(edge / sqrt(3) * 2)));
	margin_y = std::max(margin_y, edge);
	//
	// keret előállítása
	//
	typedef pair<int, int> point;
	typedef pair<point, point> line;
	list<line> lines;
	for (int y = 0; y < rows + 2; y++) {
		for (int x = 0; x < files; x++) {
			bool above = y >= 2 && fields[y-2][x];
			bool below = y < rows && fields[y][x];
			if (above != below)
				lines.push_back(line(point(3*x+1, y), point(3*x+3, y)));
		}
	}
	for (int y = 0; y <= rows; y++) {
		for (int x = 0; x <= files; x++) {
			{
				bool top_left     = x > 0 && y > 0 && fields[y-1][x-1];
				bool bottom_right = x < files && y < rows && fields[y][x];
				if (top_left != bottom_right)
					lines.push_back(line(point(3*x, y+1), point(3*x+1, y)));
			}
			{
				bool top_right   = x < files && y > 0 && fields[y-1][x];
				bool bottom_left = x > 0 && y < rows && fields[y][x-1];
				if (top_right != bottom_left)
					lines.push_back(line(point(3*x, y), point(3*x+1, y+1)));
			}
		}
	}
	//
	// kiírás
	//
	cout << "<?xml version='1.0' encoding='UTF-8' standalone='no'?>\n";
	cout << "<!--\n";
	cout << "\tA képet a hex.cpp generálta a következő paraméterekkel:\n";
	for (unsigned i = 1; i < sargs.size(); i++)
		cout << (i == 1 ? "\t" : " ") << sargs[i];
	cout << "\n";
	cout << "\tFelhasználási feltételek: Nevezd meg! - Így add tovább! (Creative Commons)\n";
	cout << "\tUray M. János\n";
	cout << "\t2013-2014\n";
	cout << "-->\n";
	cout << "<!DOCTYPE svg PUBLIC '-//W3C//DTD SVG 1.1//EN' 'http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd'>\n";
	int full_width = ((3*files + 1) * hexside + 3)/4*2 + 2*margin_x;
	int full_height = ((rows + 1) * size + 3)/4*2 + 2*margin_y;
	cout << "<svg width='" << full_width << "' height='" << full_height << "' viewBox='" << -margin_x << " " << -margin_y << " " << full_width << " " << full_height << "' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'>\n";
	if (edge) {
		cout << "\t\n";
		cout << "\t<!-- keret -->\n";
		while (!lines.empty()) {
			cout << "\t<polygon fill='none' stroke='#666' stroke-width='" << edge*2 << "' stroke-linejoin='miter' points='";
			auto line_it = lines.begin();
			line line = *line_it;
			point begin = line.first;
			point p = begin;
			do {
				cout << p.first * hexside/2 << "," << p.second * size/2 << " ";
				lines.erase(line_it);
				p = p == line.first ? line.second : line.first;
				if (p != begin) {
					auto it = lines.begin();
					while (it != lines.end() && !(it->first == p || it->second == p))
						it++;
					assert(it != lines.end());
					line_it = it;
					line = *line_it;
				}
			} while (p != begin);
			cout << "'/>\n";
		}
	}
	cout << "\t\n";
	cout << "\t<!-- mezők -->\n";
	char const* color_orders[6][3] = {
		{ "#FFF", "#CCC", "#999" },
		{ "#FFF", "#999", "#CCC" },
		{ "#CCC", "#999", "#FFF" },
		{ "#CCC", "#FFF", "#999" },
		{ "#999", "#FFF", "#CCC" },
		{ "#999", "#CCC", "#FFF" },
	};
	for (int y = 0; y < rows; y++) {
		char const* color = color_orders[color_order][y % 3];
		for (int x = 0; x < files; x++) {
			if (!fields[y][x])
				continue;
			cout << "\t<polygon fill='" << color << "' points='";
			if (y+1 < rows && x+1 < files && fields[y+1][x+1]) {
				cout << (3*x + 5) * hexside/2 << "," << (y + 2) * size/2 << " ";
			} else {
				cout << (3*x + 3) * hexside/2 << "," << (y + 2) * size/2 << " ";
				cout << (3*x + 4) * hexside/2 << "," << (y + 1) * size/2 << " ";
			}
			{
				cout << (3*x + 3) * hexside/2 << "," << (y + 0) * size/2 << " ";
				cout << (3*x + 1) * hexside/2 << "," << (y + 0) * size/2 << " ";
			}
			if (y+1 < rows && x-1 >= 0 && fields[y+1][x-1]) {
				cout << (3*x - 1) * hexside/2 << "," << (y + 2) * size/2 << " ";
			} else {
				cout << (3*x + 0) * hexside/2 << "," << (y + 1) * size/2 << " ";
				cout << (3*x + 1) * hexside/2 << "," << (y + 2) * size/2 << " ";
			}
			cout << "'/>\n";
		}
	}
	cout << "\t\n";
	cout << "</svg>\n";
}
